@@ -29,7 +29,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb.h 2272 2009-03-14 18:02:16Z teuf $
+|  $Id$
 */
 
 #ifndef __ITUNESDB_H__
@@ -106,16 +106,25 @@ typedef struct _Itdb_Chapterdata Itdb_Chapterdata;
  * @ITDB_IPOD_GENERATION_SHUFFLE_1: First Generation iPod Shuffle
  * @ITDB_IPOD_GENERATION_SHUFFLE_2: Second Generation iPod Shuffle
  * @ITDB_IPOD_GENERATION_SHUFFLE_3: Third Generation iPod Shuffle
+ * @ITDB_IPOD_GENERATION_SHUFFLE_4: Third Generation iPod Shuffle
  * @ITDB_IPOD_GENERATION_NANO_1:    First Generation iPod Nano
  * @ITDB_IPOD_GENERATION_NANO_2:    Second Generation iPod Nano
  * @ITDB_IPOD_GENERATION_NANO_3:    Third Generation iPod Nano
  * @ITDB_IPOD_GENERATION_NANO_4:    Fourth Generation iPod Nano
+ * @ITDB_IPOD_GENERATION_NANO_5:    Fifth Generation iPod Nano (with camera)
  * @ITDB_IPOD_GENERATION_VIDEO_1:   First Generation iPod Video (aka 5g)
  * @ITDB_IPOD_GENERATION_VIDEO_2:   Second Generation iPod Video (aka 5.5g)
  * @ITDB_IPOD_GENERATION_CLASSIC_1: First Generation iPod Classic
  * @ITDB_IPOD_GENERATION_CLASSIC_2: Second Generation iPod Classic
+ * @ITDB_IPOD_GENERATION_CLASSIC_3: Third Generation iPod Classic
  * @ITDB_IPOD_GENERATION_TOUCH_1:   First Generation iPod Touch
+ * @ITDB_IPOD_GENERATION_TOUCH_2:   Second Generation iPod Touch
+ * @ITDB_IPOD_GENERATION_TOUCH_3:   Third Generation iPod Touch
+ * @ITDB_IPOD_GENERATION_TOUCH_4:   Fourth Generation iPod Touch
  * @ITDB_IPOD_GENERATION_IPHONE_1:  First Generation iPhone
+ * @ITDB_IPOD_GENERATION_IPHONE_2:  Second Generation iPhone (aka iPhone 3G)
+ * @ITDB_IPOD_GENERATION_IPHONE_3:  Third Generation iPhone (aka iPhone 3GS)
+ * @ITDB_IPOD_GENERATION_IPHONE_4:  Fourth Generation iPhone
  *
  * iPod generation information
  *
@@ -147,6 +156,17 @@ typedef enum {
     ITDB_IPOD_GENERATION_CLASSIC_2,
     ITDB_IPOD_GENERATION_TOUCH_1,
     ITDB_IPOD_GENERATION_IPHONE_1,
+    ITDB_IPOD_GENERATION_SHUFFLE_4,
+    ITDB_IPOD_GENERATION_TOUCH_2,
+    ITDB_IPOD_GENERATION_IPHONE_2,
+    ITDB_IPOD_GENERATION_IPHONE_3,
+    ITDB_IPOD_GENERATION_CLASSIC_3,
+    ITDB_IPOD_GENERATION_NANO_5,
+    ITDB_IPOD_GENERATION_TOUCH_3,
+    ITDB_IPOD_GENERATION_IPAD_1,
+    ITDB_IPOD_GENERATION_IPHONE_4,
+    ITDB_IPOD_GENERATION_TOUCH_4,
+    ITDB_IPOD_GENERATION_NANO_6
 } Itdb_IpodGeneration;
 
 /**
@@ -179,6 +199,7 @@ typedef enum {
  * @ITDB_IPOD_MODEL_NANO_ORANGE:    iPod Nano (Orange)
  * @ITDB_IPOD_MODEL_IPHONE_1:       iPhone
  * @ITDB_IPOD_MODEL_SHUFFLE_SILVER: iPod Shuffle (Silver)
+ * @ITDB_IPOD_MODEL_SHUFFLE_BLACK:  iPod Shuffle (Black)
  * @ITDB_IPOD_MODEL_SHUFFLE_PINK:   iPod Shuffle (Pink)
  * @ITDB_IPOD_MODEL_SHUFFLE_BLUE:   iPod Shuffle (Blue)
  * @ITDB_IPOD_MODEL_SHUFFLE_GREEN:  iPod Shuffle (Green)
@@ -187,7 +208,9 @@ typedef enum {
  * @ITDB_IPOD_MODEL_SHUFFLE_RED:    iPod Shuffle (Red)
  * @ITDB_IPOD_MODEL_CLASSIC_SILVER: iPod Classic (Silver)
  * @ITDB_IPOD_MODEL_CLASSIC_BLACK:  iPod Classic (Black)
- * @ITDB_IPOD_MODEL_TOUCH_BLACK:    iPod Touch (Black)
+ * @ITDB_IPOD_MODEL_TOUCH_SILVER:   iPod Touch (Silver)
+ * @ITDB_IPOD_MODEL_IPHONE_WHITE:   iPhone (White)
+ * @ITDB_IPOD_MODEL_IPHONE_BLACK:   iPhone (Black)
  *
  * iPod model information
  *
@@ -230,7 +253,13 @@ typedef enum {
     ITDB_IPOD_MODEL_SHUFFLE_RED,
     ITDB_IPOD_MODEL_CLASSIC_SILVER,
     ITDB_IPOD_MODEL_CLASSIC_BLACK,
-    ITDB_IPOD_MODEL_TOUCH_BLACK,
+    ITDB_IPOD_MODEL_TOUCH_SILVER,
+    ITDB_IPOD_MODEL_SHUFFLE_BLACK,
+    ITDB_IPOD_MODEL_IPHONE_WHITE,
+    ITDB_IPOD_MODEL_IPHONE_BLACK,
+    ITDB_IPOD_MODEL_SHUFFLE_GOLD,
+    ITDB_IPOD_MODEL_SHUFFLE_STAINLESS,
+    ITDB_IPOD_MODEL_IPAD
 } Itdb_IpodModel;
 
 /**
@@ -427,6 +456,8 @@ typedef enum {
  *  bit 7 = unknown, but probably less than or equal to
  *  bit 8 = a range selection
  *  bit 9 = "in the last"
+ *  bit 10 = binary AND
+ *  bit 11 = unknown
  *  </programlisting>
  * </informalexample>
  *
@@ -439,6 +470,20 @@ typedef enum {
     ITDB_SPLACTION_IS_IN_THE_RANGE = 0x00000100,
     ITDB_SPLACTION_IS_IN_THE_LAST = 0x00000200,
     ITDB_SPLACTION_BINARY_AND = 0x00000400,
+    /* This action has been seen in the smart playlists stored in mhsd5
+     * on recent ipods. It operates on the "video kind" field. 
+     * It uses integer values, and the from/to values are different.
+     * The "from" value might be the list of bits that are allowed to be set
+     * in the value we are matching, and the "to" value might be a value
+     * that must be set in the value we are matching. For example, there's 
+     * a TVShow playlist using that action, from is 0x208044, to is 0x40. 
+     * "video kind" is a bitfield so the 1st value let us filter out
+     * entries with unwanted bits, and the second one ensures we have a
+     * TVShow. A match could be tested with:
+     * is_match = (((val & from) == val) && (val & to))
+     * I'm not sure what this becomes when this action is negated...
+     */
+    ITDB_SPLACTION_BINARY_UNKNOWN1 = 0x00000800,
 
     ITDB_SPLACTION_IS_STRING = 0x01000001,
     ITDB_SPLACTION_CONTAINS = 0x01000002,
@@ -450,6 +495,9 @@ typedef enum {
     ITDB_SPLACTION_IS_NOT_LESS_THAN = 0x02000040,
     ITDB_SPLACTION_IS_NOT_IN_THE_RANGE = 0x02000100,
     ITDB_SPLACTION_IS_NOT_IN_THE_LAST = 0x02000200,
+    ITDB_SPLACTION_NOT_BINARY_AND = 0x02000400,
+    ITDB_SPLACTION_BINARY_UNKNOWN2 = 0x02000800,
+
 
     ITDB_SPLACTION_IS_NOT = 0x03000001,
     ITDB_SPLACTION_DOES_NOT_CONTAIN = 0x03000002,
@@ -626,12 +674,23 @@ typedef enum {
     ITDB_SPLFIELD_BPM = 0x23,
     ITDB_SPLFIELD_GROUPING = 0x27,
     ITDB_SPLFIELD_PLAYLIST = 0x28,
+    ITDB_SPLFIELD_PURCHASE = 0x29,
+    ITDB_SPLFIELD_DESCRIPTION = 0x36,
+    ITDB_SPLFIELD_CATEGORY = 0x37,
+    ITDB_SPLFIELD_PODCAST = 0x39,
     ITDB_SPLFIELD_VIDEO_KIND = 0x3c,
     ITDB_SPLFIELD_TVSHOW = 0x3e,
     ITDB_SPLFIELD_SEASON_NR = 0x3f,
     ITDB_SPLFIELD_SKIPCOUNT = 0x44,
     ITDB_SPLFIELD_LAST_SKIPPED = 0x45,
-    ITDB_SPLFIELD_ALBUMARTIST = 0x47
+    ITDB_SPLFIELD_ALBUMARTIST = 0x47,
+    ITDB_SPLFIELD_SORT_SONG_NAME = 0x4e,
+    ITDB_SPLFIELD_SORT_ALBUM = 0x4f,
+    ITDB_SPLFIELD_SORT_ARTIST = 0x50,
+    ITDB_SPLFIELD_SORT_ALBUMARTIST = 0x51,
+    ITDB_SPLFIELD_SORT_COMPOSER = 0x52,
+    ITDB_SPLFIELD_SORT_TVSHOW = 0x53,
+    ITDB_SPLFIELD_ALBUM_RATING = 0x5a
 } ItdbSPLField;
 
 /**
@@ -916,6 +975,7 @@ struct _Itdb_PhotoDB
     ItdbUserDataDestroyFunc userdata_destroy;
 };
 
+typedef struct _Itdb_iTunesDB_Private Itdb_iTunesDB_Private;
 /**
  * Itdb_iTunesDB:
  * @tracks:             A list of tracks in the database (#Itdb_Track)
@@ -924,9 +984,9 @@ struct _Itdb_PhotoDB
  * @device:             iPod device info (#Itdb_Device)
  * @version:            The version number of the iTunesDB
  * @id:                 A 64 bit id value for the iTunesDB
- * @reserved_int1:      Reserved for future use
+ * @tzoffset:           offset in seconds from UTC
  * @reserved_int2:      Reserved for future use
- * @reserved1:          Reserved for future use
+ * @priv:               Private data
  * @reserved2:          Reserved for future use
  * @usertype:           For use by application
  * @userdata:           For use by application
@@ -943,10 +1003,10 @@ struct _Itdb_iTunesDB
     Itdb_Device *device;
     guint32 version;
     guint64 id;
+    gint32 tzoffset;
     /* reserved for future use */
-    gint32 reserved_int1;
     gint32 reserved_int2;
-    gpointer reserved1;
+    Itdb_iTunesDB_Private *priv;
     gpointer reserved2;
     /* below is for use by application */
     guint64 usertype;
@@ -977,8 +1037,8 @@ struct _Itdb_iTunesDB
  * @unk028:               Unknown, seems to be always 0
  * @unk044:               Unknown, seems to always be 0
  * @unk048:               Unknown, seems to always be 0
- * @album_id:             The id of the album.  This is set automatically when
- *                        the PhotoDB is written.
+ * @album_id:             Unique integer for each playlist. This is set
+ *                        automatically when the PhotoDB is written.
  * @prev_album_id:        The id of the previous playlist.  This is set
  *                        automatically when the PhotoDB is written.
  * @reserved_int1:        Reserved for future use
@@ -1028,6 +1088,7 @@ struct _Itdb_PhotoAlbum
     ItdbUserDataDestroyFunc userdata_destroy;
 };
 
+typedef struct _Itdb_Playlist_Private Itdb_Playlist_Private;
 /**
  * Itdb_Playlist:
  * @itdb:               A pointer to the #Itdb_iTunesDB (for convenience)
@@ -1055,7 +1116,7 @@ struct _Itdb_PhotoAlbum
  * @reserved101:        Reserved for MHOD100 implementation
  * @reserved_int1:      Reserved for future use
  * @reserved_int2:      Reserved for future use
- * @reserved1:          Reserved for future use
+ * @priv:               Private data
  * @reserved2:          Reserved for future use
  * @usertype:           For use by application
  * @userdata:           For use by application
@@ -1086,7 +1147,7 @@ struct _Itdb_Playlist
     /* reserved for future use */
     gint32 reserved_int1;
     gint32 reserved_int2;
-    gpointer reserved1;
+    Itdb_Playlist_Private *priv;
     gpointer reserved2;
     /* below is for use by application */
     guint64 usertype;
@@ -1103,7 +1164,7 @@ struct _Itdb_Playlist
  * @ITDB_PSO_TITLE:         Sort by track title
  * @ITDB_PSO_ALBUM:         Sort by album
  * @ITDB_PSO_ARTIST:        Sort by artist
- * @ITDB_PSO_BIRATE:        Sort by bitrate
+ * @ITDB_PSO_BITRATE:       Sort by bitrate
  * @ITDB_PSO_GENRE:         Sort by genre
  * @ITDB_PSO_FILETYPE:      Sort by filetype
  * @ITDB_PSO_TIME_MODIFIED: Sort by date modified
@@ -1137,7 +1198,7 @@ typedef enum
     ITDB_PSO_TITLE = 3,
     ITDB_PSO_ALBUM = 4,
     ITDB_PSO_ARTIST = 5,
-    ITDB_PSO_BIRATE = 6,
+    ITDB_PSO_BITRATE = 6,
     ITDB_PSO_GENRE = 7,
     ITDB_PSO_FILETYPE = 8,
     ITDB_PSO_TIME_MODIFIED = 9,
@@ -1161,6 +1222,11 @@ typedef enum
     ITDB_PSO_CATEGORY = 27,
     ITDB_PSO_DESCRIPTION = 28
 } ItdbPlaylistSortOrder;
+/* libgpod 0.7.90 and earlier had a typo in ITDB_PSO_BITRATE, workaround
+ * that typo and avoid breaking the API (even if I doubt there are users 
+ * of this)
+ */
+#define ITDB_PSO_BIRATE ITDB_PSO_BITRATE
 
 
 /**
@@ -1168,28 +1234,52 @@ typedef enum
  * @ITDB_MEDIATYPE_AUDIO:      Audio files
  * @ITDB_MEDIATYPE_MOVIE:      Movies
  * @ITDB_MEDIATYPE_PODCAST:    Podcasts
+ * @ITDB_MEDIATYPE_VIDEO_PODCAST:    Video Podcasts
  * @ITDB_MEDIATYPE_AUDIOBOOK:  Audio books
  * @ITDB_MEDIATYPE_MUSICVIDEO: Music videos
  * @ITDB_MEDIATYPE_TVSHOW:     TV Shows
+ * @ITDB_MEDIATYPE_MUSIC_TVSHOW:     TV Shows (also show in Music)
+ * @ITDB_MEDIATYPE_RINGTONE:   Ringtone
+ * @ITDB_MEDIATYPE_RENTAL:     Rental
+ * @ITDB_MEDIATYPE_ITUNES_EXTRA: ?
+ * @ITDB_MEDIATYPE_MEMO:       Memo
+ * @ITDB_MEDIATYPE_ITUNES_U:   iTunes U
  *
  * Mediatype definitions
  *
  * The mediatype is used to determine what menu a track appears under.  For
  * example, setting the mediatype to #ITDB_MEDIATYPE_PODCAST makes the track
- * appear on the Podcast menu.
+ * appear on the Podcast menu. Media type is a bitfield, so it can be a
+ * binary combination of these constants, make sure to use binary operators
+ * when you want to operate on media types (eg use a binary AND in
+ * preference over a straight == when you want to test if a track has a
+ * given media type).
  *
  * Since: 0.5.0
  */
 typedef enum
 {
-    ITDB_MEDIATYPE_AUDIO      = 0x0001,
-    ITDB_MEDIATYPE_MOVIE      = 0x0002,
-    ITDB_MEDIATYPE_PODCAST    = 0x0004,
-    ITDB_MEDIATYPE_AUDIOBOOK  = 0x0008,
-    ITDB_MEDIATYPE_MUSICVIDEO = 0x0020,
-    ITDB_MEDIATYPE_TVSHOW     = 0x0040,
+    ITDB_MEDIATYPE_AUDIO        = (1 << 0),
+    ITDB_MEDIATYPE_MOVIE        = (1 << 1),
+    ITDB_MEDIATYPE_PODCAST      = (1 << 2),
+    ITDB_MEDIATYPE_AUDIOBOOK    = (1 << 3),
+    ITDB_MEDIATYPE_MUSICVIDEO   = (1 << 5),
+    ITDB_MEDIATYPE_TVSHOW       = (1 << 6),
+    ITDB_MEDIATYPE_RINGTONE     = (1 << 14),
+    ITDB_MEDIATYPE_RENTAL       = (1 << 15),
+    ITDB_MEDIATYPE_ITUNES_EXTRA = (1 << 16),
+    ITDB_MEDIATYPE_MEMO         = (1 << 20),
+    ITDB_MEDIATYPE_ITUNES_U     = (1 << 21),
+    ITDB_MEDIATYPE_EPUB_BOOK    = (1 << 22)
 } Itdb_Mediatype;
 
+/* Known compound media types which have been observed in iPod databases.
+ * This list is in no way exhaustive, and these constants are only helpers,
+ * there's nothing wrong with not using them.
+ */
+#define ITDB_MEDIATYPE_VIDEO_PODCAST (ITDB_MEDIATYPE_MOVIE | ITDB_MEDIATYPE_PODCAST)
+#define ITDB_MEDIATYPE_MUSIC_TVSHOW (ITDB_MEDIATYPE_MUSICVIDEO | ITDB_MEDIATYPE_TVSHOW)
+typedef struct _Itdb_Track_Private Itdb_Track_Private;
 /**
  * Itdb_Track:
  * @itdb:                       A pointer to the #Itdb_iTunesDB (for convenience)
@@ -1302,9 +1392,9 @@ typedef enum
  * @type2:                      MP3s are 0x01, AAC are 0x00
  * @compilation:                Flag to mark the track as a compilation.  True
  *                              if set to 0x1, false if set to 0x0.
- * @starttime:                  The time, in milliseconds, at which the song
+ * @starttime:                  The offset, in milliseconds, at which the song
  *                              will start playing.
- * @stoptime:                   The time, in milliseconds, at which the song
+ * @stoptime:                   The offset, in milliseconds, at which the song
  *                              will stop playing.
  * @checked:                    Flag for whether the track is checked.  True if
  *                              set to 0x0, false if set to 0x1
@@ -1483,6 +1573,7 @@ typedef enum
  * @userdata:                   For use by application
  * @userdata_duplicate:         A function to duplicate #userdata
  * @userdata_destroy:           A function to free #userdata
+ * @priv:                       Private data
  *
  * Structure representing a track in an iTunesDB
  *
@@ -1610,7 +1701,7 @@ struct _Itdb_Track
   guint32 unk252;
   guint16 gapless_track_flag;
   guint16 gapless_album_flag;
-  guint16 album_id;
+  guint16 obsolete;
 
   /* This is for Cover Art support */
   struct _Itdb_Artwork *artwork;
@@ -1625,7 +1716,7 @@ struct _Itdb_Track
   gint32 reserved_int4;
   gint32 reserved_int5;
   gint32 reserved_int6;
-  gpointer reserved1;
+  Itdb_Track_Private *priv;
   gpointer reserved2;
   gpointer reserved3;
   gpointer reserved4;
@@ -1647,7 +1738,6 @@ struct _Itdb_Track
 /* (gtkpod note: don't forget to add fields read from the file to
  * copy_new_info() in file.c!) */
 
-
 /* ------------------------------------------------------------ *\
  *
  * Error codes
@@ -1673,9 +1763,20 @@ typedef enum
     ITDB_FILE_ERROR_ITDB_CORRUPT
 } ItdbFileError;
 
+typedef enum
+{
+    ITDB_ERROR_SEEK,
+    ITDB_ERROR_CORRUPT,
+    ITDB_ERROR_NOTFOUND,
+    ITDB_ERROR_RENAME,
+    ITDB_ERROR_ITDB_CORRUPT,
+    ITDB_ERROR_SQLITE
+} ItdbError;
+
 
 /* Error domain */
-#define ITDB_FILE_ERROR itdb_file_error_quark ()
+#define ITDB_ERROR itdb_file_error_quark ()
+#define ITDB_FILE_ERROR ITDB_ERROR
 GQuark     itdb_file_error_quark      (void);
 
 
@@ -1694,6 +1795,8 @@ gboolean itdb_write_file (Itdb_iTunesDB *itdb, const gchar *filename,
 gboolean itdb_shuffle_write (Itdb_iTunesDB *itdb, GError **error);
 gboolean itdb_shuffle_write_file (Itdb_iTunesDB *itdb,
 				  const gchar *filename, GError **error);
+gboolean itdb_start_sync (Itdb_iTunesDB *itdb);
+gboolean itdb_stop_sync (Itdb_iTunesDB *itdb);
 Itdb_iTunesDB *itdb_new (void);
 void itdb_free (Itdb_iTunesDB *itdb);
 Itdb_iTunesDB *itdb_duplicate (Itdb_iTunesDB *itdb); /* not implemented */
@@ -1731,6 +1834,7 @@ gchar *itdb_get_photos_dir (const gchar *mountpoint);
 gchar *itdb_get_photos_thumb_dir (const gchar *mountpoint);
 gchar *itdb_get_device_dir (const gchar *mountpoint);
 gchar *itdb_get_itunesdb_path (const gchar *mountpoint);
+gchar *itdb_get_itunescdb_path (const gchar *mountpoint);
 gchar *itdb_get_itunessd_path (const gchar *mountpoint);
 gchar *itdb_get_artworkdb_path (const gchar *mountpoint);
 gchar *itdb_get_photodb_path (const gchar *mountpoint);
@@ -1754,6 +1858,7 @@ gboolean itdb_device_supports_photo (const Itdb_Device *device);
 gboolean itdb_device_supports_podcast (const Itdb_Device *device);
 const gchar *itdb_info_get_ipod_model_name_string (Itdb_IpodModel model);
 const gchar *itdb_info_get_ipod_generation_string (Itdb_IpodGeneration generation);
+gchar *itdb_device_get_uuid(const Itdb_Device *device);
 
 /* track functions */
 Itdb_Track *itdb_track_new (void);
@@ -1771,7 +1876,7 @@ Itdb_Track *itdb_track_id_tree_by_id (GTree *idtree, guint32 id);
 Itdb_Playlist *itdb_playlist_new (const gchar *title, gboolean spl);
 void itdb_playlist_free (Itdb_Playlist *pl);
 void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos);
-void itdb_playlist_move (Itdb_Playlist *pl, guint32 pos);
+void itdb_playlist_move (Itdb_Playlist *pl, gint32 pos);
 void itdb_playlist_remove (Itdb_Playlist *pl);
 void itdb_playlist_unlink (Itdb_Playlist *pl);
 Itdb_Playlist *itdb_playlist_duplicate (Itdb_Playlist *pl);
@@ -1797,6 +1902,9 @@ Itdb_Playlist *itdb_playlist_podcasts (Itdb_iTunesDB *itdb);
 gboolean itdb_playlist_is_podcasts (Itdb_Playlist *pl);
 void itdb_playlist_set_podcasts (Itdb_Playlist *pl);
 
+/* playlist functions for audiobooks playlist */
+gboolean itdb_playlist_is_audiobooks (Itdb_Playlist *pl);
+
 /* smart playlist functions */
 ItdbSPLFieldType itdb_splr_get_field_type (const Itdb_SPLRule *splr);
 ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr);
@@ -1849,6 +1957,9 @@ Itdb_PhotoAlbum *itdb_photodb_photoalbum_create (Itdb_PhotoDB *db,
 						 const gchar *albumname,
 						 gint pos);
 Itdb_PhotoDB *itdb_photodb_create (const gchar *mountpoint);
+Itdb_PhotoAlbum *itdb_photodb_photoalbum_new (const gchar *albumname);
+void itdb_photodb_photoalbum_free (Itdb_PhotoAlbum *album);
+void itdb_photodb_photoalbum_add (Itdb_PhotoDB *db, Itdb_PhotoAlbum *album, gint pos);
 void itdb_photodb_free (Itdb_PhotoDB *photodb);
 gboolean itdb_photodb_write (Itdb_PhotoDB *photodb, GError **error);
 void itdb_photodb_remove_photo (Itdb_PhotoDB *db,
@@ -1857,6 +1968,7 @@ void itdb_photodb_remove_photo (Itdb_PhotoDB *db,
 void itdb_photodb_photoalbum_remove (Itdb_PhotoDB *db,
 				     Itdb_PhotoAlbum *album,
 				     gboolean remove_pics);
+void itdb_photodb_photoalbum_unlink (Itdb_PhotoAlbum *album);
 Itdb_PhotoAlbum *itdb_photodb_photoalbum_by_name(Itdb_PhotoDB *db,
 						 const gchar *albumname );
 
@@ -1896,12 +2008,13 @@ Itdb_Chapterdata *itdb_chapterdata_new (void);
 void itdb_chapterdata_free (Itdb_Chapterdata *chapterdata);
 Itdb_Chapterdata *itdb_chapterdata_duplicate (Itdb_Chapterdata *chapterdata);
 void itdb_chapterdata_remove_chapter (Itdb_Chapterdata *chapterdata, Itdb_Chapter *chapter);
+void itdb_chapterdata_unlink_chapter (Itdb_Chapterdata *chapterdata, Itdb_Chapter *chapter);
 void itdb_chapterdata_remove_chapters (Itdb_Chapterdata *chapterdata);
 Itdb_Chapter *itdb_chapter_new (void);
 void itdb_chapter_free (Itdb_Chapter *chapter);
 Itdb_Chapter *itdb_chapter_duplicate (Itdb_Chapter *chapter);
 gboolean itdb_chapterdata_add_chapter (Itdb_Chapterdata *chapterdata,
-				       gint32 startpos,
+				       guint32 startpos,
 				       gchar *chaptertitle);
 
 #ifndef LIBGPOD_DISABLE_DEPRECATED
